@@ -1850,7 +1850,13 @@ osi_bool osi_config_parse_non_globals( OSI_SCAN_CONFIG *cfg )
             case CONFIG_KEYWORD_NOENTRY:
 
                 osi_strlcpy( temp, current_block->directory, sizeof( temp ) );
-                osi_strlcat( temp, path_separator, sizeof( temp ) );
+
+                if ( ( strlen(temp) > 0 ) &&
+                     ( temp[strlen(temp)-1] != PATH_SEPARATOR ) )
+                {
+                    osi_strlcat( temp, path_separator, sizeof( temp ) );
+                }
+
                 osi_strlcat( temp, line, sizeof( temp ) );
 
                 string_list_add_item( current_block->no_entry_list, temp );
