@@ -785,18 +785,28 @@ void compare_db_with_default_for_host( OSI_DB *db, OSI_HOST *host )
             if( ( host->notify_flags & OSI_NOTIFY_SCAN_ALWAYS ) ||
                 ( context->diff_count > 0 ) )
             {
+                osi_bool show_url = !( host->db_flags & OSI_DB_AUTOACCEPT );  
+
                 if( strlen( host->notify_email ) > 0 )
                 {
-                    md_notify_admin( host->notify_email, context );
+                    md_notify_admin( host->notify_email, context, show_url );
                 }
 
                 else
                 {
-                    md_notify_admin( config->notify_email, context );
+                    md_notify_admin( config->notify_email, context, show_url );
                 }
             }
         }
 
+        /* ALWAYS send notification to admin email. */
+
+        else if ( strlen( config->admin_email ) )
+        {
+            osi_bool show_url = !( host->db_flags & OSI_DB_AUTOACCEPT );
+            md_notify_admin( config->admin_email, context, show_url );
+        }
+
         osi_db_ctx_destroy( context );
     }
 
