@@ -259,6 +259,9 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
     char buf[OSIRIS_SMTP_CMD_BUFSZ];
     OSI_ERROR err;
 
+    int notify_email = 1;
+    int admin_email  = 1;
+
     /* no error passed in, use a local one. */
 
     if( error == NULL )
@@ -266,10 +269,21 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
         error = &err;
     }
 
-    if( email == NULL )
+    if ( ( email == NULL ) || ( strlen( email ) == 0 ) )
+    {
+        notify_email = 0;
+    }
+
+    if ( ( config->admin_email == NULL ) ||
+         ( strlen( config->admin_email ) == 0 ) )
+    {
+        admin_email = 0;
+    } 
+
+    if ( notify_email == 0 && admin_email == 0 )
     {
         osi_strlcpy( error->message,
-                     "notification failure: recipient email address not set.",
+                     "notification failure: no admin or notify address set!!.",
                      sizeof( error->message ) );
 
         log_error( LOG_ID_NOTIFY_ERROR, NULL, error->message );
@@ -330,15 +344,50 @@ static int connect_to_mail_server( const char *email, OSI_ERROR *error )
             break;
         }
 
-        /* MAIL - sender uses same addresss as recipient */
-        osi_snprintf(buf, OSIRIS_SMTP_CMD_BUFSZ, "MAIL FROM: <%s>\r\n", email);
-        if (send_smtp_command(sock, buf, "250"))
-            break;
+        if ( notify_email )
+        {
+            /* MAIL - sender uses same addresss as recipient */
+            osi_snprintf(buf, OSIRIS_SMTP_CMD_BUFSZ,
+                         "MAIL FROM: <%s>\r\n", email);
 
-        /* RCPT */
-        osi_snprintf(buf, OSIRIS_SMTP_CMD_BUFSZ, "RCPT TO: <%s>\r\n", email);
-        if (send_smtp_command(sock, buf, "250"))
-            break;
+            if (send_smtp_command(sock, buf, "250"))
+            {
+                break;
+            }
+
+            /* RCPT */
+            osi_snprintf(buf, OSIRIS_SMTP_CMD_BUFSZ,
+                         "RCPT TO: <%s>\r\n", email);
+
+            if (send_smtp_command(sock, buf, "250"))
+            {
+                break;
+            }
+        }
+
+        /* always send to admin. */
+
+        if ( admin_email )
+        {
+            if ( !notify_email )
+            {            
+                osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ,
+                              "MAIL FROM: <%s>\r\n", config->admin_email );
+
+                if ( send_smtp_command( sock, buf, "250" ) )
+                {
+                    break;
+                }
+            }
+
+            osi_snprintf( buf, OSIRIS_SMTP_CMD_BUFSZ, "RCPT TO: <%s>\r\n",
+                          config->admin_email );
+
+            if ( send_smtp_command( sock, buf, "250" ) )
+            {
+                break;
+            }
+        }
 
         /* DATA */
         if( send_smtp_command( sock, OSIRIS_SMTP_CMD_DATA, "354" ) )
@@ -358,6 +407,8 @@ void md_notify_admin_session_rekey( const char *host, const char *email )
     char buf[OSIRIS_SMTP_CMD_BUFSZ];
     char this_host[MAX_HOSTNAME_LENGTH];
 
+    const char *from_email = email;
+
     if( host == NULL )
     {
         log_error( LOG_ID_NOTIFY_ERROR, "unknown", "null host" );
@@ -369,6 +420,14 @@ void md_notify_admin_session_rekey( const char *host, const char *email )
         return;
     }
 
+    /* make sure we have a valid email address, after the connect, */
+    /* at least one should be valid.                               */
+
+    if ( email && strlen( email ) == 0 )
+    {
+        from_email = config->admin_email;
+    }
+
     if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
     {
         osi_snprintf( this_host, sizeof(this_host), "%s", config->http_host );
@@ -390,7 +449,8 @@ void md_notify_admin_session_rekey( const char *host, const char *email )
                   "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
                   "Date: %s\r\n"
                   "Subject: [host: %s] session rekey.\r\n\r\n",
-                  email, this_host, osi_time_to_string(osi_get_time()), host );
+                  from_email, this_host,
+                  osi_time_to_string(osi_get_time()), host );
 
     send_smtp_command( sock, buf, NULL );
 
@@ -415,6 +475,8 @@ void md_notify_admin_scan_failed( const char *host,
     char buf[OSIRIS_SMTP_CMD_BUFSZ];
     char this_host[MAX_HOSTNAME_LENGTH];
 
+    const char *from_email = email;
+
     if( host == NULL )
     {
         log_error( LOG_ID_NOTIFY_ERROR, "unknown", "null host" );
@@ -438,6 +500,11 @@ void md_notify_admin_scan_failed( const char *host,
         return;
     }
 
+    if ( email && strlen( email ) == 0 )
+    {
+        from_email = config->admin_email;
+    }
+
     if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
     {
         osi_snprintf( this_host, sizeof(this_host), "%s", config->http_host );
@@ -459,7 +526,8 @@ void md_notify_admin_scan_failed( const char *host,
                   "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
                   "Date: %s\r\n"
              "Subject: [host: %s] failed to start the scheduled scan.\r\n\r\n",
-                  email, this_host, osi_time_to_string(osi_get_time()), host );
+                  from_email, this_host,
+                  osi_time_to_string(osi_get_time()), host );
 
     send_smtp_command( sock, buf, NULL );
 
@@ -475,7 +543,7 @@ void md_notify_admin_scan_failed( const char *host,
     return;
 }
 
-void md_notify_admin( const char *email, OSI_DB_CTX *context )
+void md_notify_admin( const char *email, OSI_DB_CTX *context,osi_bool show_url )
 {
     int sock = 0;
     int diff_count = context->diff_count;
@@ -487,12 +555,7 @@ void md_notify_admin( const char *email, OSI_DB_CTX *context )
     char buf[OSIRIS_SMTP_CMD_BUFSZ];    /* KISS !malloc */
     char host[MAX_HOSTNAME_LENGTH];
 
-    if( ( email == NULL ) || ( strlen( email ) == 0 ) )
-    {
-        log_error( LOG_ID_NOTIFY_ERROR, context->host,
-                  "notification failure: recipient email address not set.");
-        return;
-    }
+    const char *from_email = email;
 
     if (logfile == NULL)
     {
@@ -501,16 +564,21 @@ void md_notify_admin( const char *email, OSI_DB_CTX *context )
         return;
     }
 
-    log_info( LOG_ID_NOTIFY_ERROR, context->host,
-             "notifying %s using %s:%d, sending contents of log file: %s, "
-             "containing %d entries.",
-             email, smtp_host, smtp_port,logfile, diff_count);
-
     if ( ( sock = connect_to_mail_server( email, NULL ) ) < 0  )
     {
         return;
     }
 
+    if ( email && strlen( email ) == 0 )
+    {
+        from_email = config->admin_email;
+    }
+
+   log_info( LOG_ID_NOTIFY_ERROR, context->host,
+            "notifying %s using %s:%d, sending contents of log file: %s, "
+            "containing %d entries.",
+            from_email, smtp_host, smtp_port,logfile, diff_count);
+
     /* BODY */
 
     if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
@@ -534,7 +602,7 @@ void md_notify_admin( const char *email, OSI_DB_CTX *context )
                   "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
                   "Date: %s\r\n"
                   "Subject: [host: %s][%d changes]\r\n\r\n",
-                  email, host, osi_time_to_string( osi_get_time() ),
+                  from_email, host, osi_time_to_string( osi_get_time() ),
                   context->host, diff_count );
 
     send_smtp_command( sock, buf, NULL );
@@ -543,7 +611,7 @@ void md_notify_admin( const char *email, OSI_DB_CTX *context )
     /* the recipient to approve of these changes by updating  */
     /* the trusted database via https.                        */
 
-    if( config->http_port > 0 )
+    if( ( config->http_port > 0 ) && ( show_url ) )
     {
         char filename[MAX_PATH_LENGTH] = "";
 
@@ -584,6 +652,8 @@ osi_bool md_notify_send_test( const char *email, OSI_ERROR *error )
     char buf[OSIRIS_SMTP_CMD_BUFSZ];
     char this_host[MAX_HOSTNAME_LENGTH];
 
+    const char *from_email = email;
+
     if( error == NULL )
     {
         return FALSE;
@@ -596,6 +666,11 @@ osi_bool md_notify_send_test( const char *email, OSI_ERROR *error )
         return FALSE;
     }
 
+    if ( email && strlen( email ) == 0 )
+    {
+        from_email = config->admin_email;
+    }
+
     if( config->http_host[0] && strcmp( config->http_host, "" ) != 0 )
     {
         osi_snprintf( this_host, sizeof(this_host), "%s", config->http_host );
@@ -615,9 +690,9 @@ osi_bool md_notify_send_test( const char *email, OSI_ERROR *error )
                   */
                   "To: %s\r\n"
                   "From: \"Osiris Host Integrity System\" <osirismd@%s>\r\n"
-                  "Date: %s\r\n\r\n"
+                  "Date: %s\r\n"
                   "Subject: [osiris test message]\r\n\r\n",
-                   email, this_host, osi_time_to_string( osi_get_time() ) );
+                  from_email, this_host, osi_time_to_string( osi_get_time() ) );
 
      send_smtp_command( sock, buf, NULL );
 
