@@ -27,7 +27,7 @@
 #ifndef MD_NOTIFY_H_
 #define MD_NOTIFY_H_
 
-void md_notify_admin( const char *email, OSI_DB_CTX *context );
+void md_notify_admin( const char *email, OSI_DB_CTX *context,osi_bool show_url);
 
 void md_notify_admin_session_rekey( const char *host, const char *email );
 
