@@ -2023,7 +2023,7 @@ osi_bool generate_keypair()
     X509 *x509     = NULL;
     EVP_PKEY *pkey = NULL;
 
-    if( make_certificate( &x509, &pkey, OSIRISMD_RSA_KEY_SIZE, 0, 365 ) )
+    if( make_certificate( &x509, &pkey, OSIRISMD_RSA_KEY_SIZE, 0, (365*10) ) )
     {
         log_info( LOG_ID_DAEMON_INFO, NULL,
                   "self-signed certificate created." );
